@@ -106,10 +106,11 @@ mongo_collection_insert_bson <- function(col, doc, stop_on_error = TRUE){
 }
 
 #' @useDynLib mongolite R_mongo_collection_update
-mongo_collection_update <- function(col, selector, update, filters, upsert = TRUE, multiple = TRUE){
+mongo_collection_update <- function(col, selector, update, filters = NULL, upsert = FALSE, multiple = FALSE, replace = FALSE){
   stopifnot(is.logical(upsert))
   stopifnot(is.logical(multiple))
-  reply <- .Call(R_mongo_collection_update, col, bson_or_json(selector), bson_or_json(update), bson_or_json(filters), upsert, multiple)
+  filters <- bson_or_json(filters, allowNull = TRUE)
+  reply <- .Call(R_mongo_collection_update, col, bson_or_json(selector), bson_or_json(update), filters, upsert, multiple, replace)
   structure(reply, class = c("miniprint"))
 }
 
