@@ -19,8 +19,10 @@
 #define BSON_HAVE_STRNLEN 1
 #define BSON_HAVE_ATOMIC_32_ADD_AND_FETCH 1
 #define BSON_HAVE_ATOMIC_64_ADD_AND_FETCH 1
+#if !defined (__FreeBSD__) && !defined (__OpenBSD__)
 #define BSON_HAVE_SYSCALL_TID 1
 #endif
+#endif
 
 /* Fix for snow leopard */
 #ifdef __APPLE__
@@ -50,6 +52,7 @@
 #else
 #define BSON_OS 1
 #define BSON_HAVE_CLOCK_GETTIME 1
+#define BSON_HAVE_RAND_R 1
 #endif
 
 /*
@@ -104,13 +107,14 @@
 # undef BSON_HAVE_SNPRINTF
 #endif
 
-
 /*
- * Define to 1 if you have _set_output_format (VS2013 and older).
+ * Define to 1 if you have gmtime_r available on your platform.
  */
-#define BSON_NEEDS_SET_OUTPUT_FORMAT 0
-#if BSON_NEEDS_SET_OUTPUT_FORMAT != 1
-# undef BSON_NEEDS_SET_OUTPUT_FORMAT
+#ifndef _WIN32
+#define BSON_HAVE_GMTIME_R 1
+#endif
+#if BSON_HAVE_GMTIME_R != 1
+# undef BSON_HAVE_GMTIME_R
 #endif
 
 /*
@@ -121,22 +125,4 @@
 # undef BSON_HAVE_TIMESPEC
 #endif
 
-
-/*
-* Define to 1 if _Decimal128 (BID format) is available on your platform.
-*/
-#define BSON_HAVE_DECIMAL128 0
-#if BSON_HAVE_DECIMAL128 != 1
-# undef BSON_HAVE_DECIMAL128
-#endif
-
-
-/*
-* Define to 1 to support experimental future BSON or MongoDB features.
-*/
-#define BSON_EXPERIMENTAL_FEATURES 0
-#if BSON_EXPERIMENTAL_FEATURES != 1
-# undef BSON_EXPERIMENTAL_FEATURES
-#endif
-
 #endif /* BSON_CONFIG_H */
